import { setSessionId, setToken } from "@/auth/auth";
import { RoleCode } from "@/types";
import { defineStore } from "pinia";

// main is the name of the store. It is unique across your application
// and will appear in devtools
const useMainStore = defineStore("main", {
  // a function that returns a fresh state
  state: () => {
    return {
      globalMask: false,
      userInfo: {
        userId: -1,
        identity: "",
        displayName: "",
        rootOrgId: -1,
        rootOrgName: "",
        token: "",
        role: "" as RoleCode,
      },
      currentLocation: "",
      paperAnalysisDetail: {
        projectName: "",
        paperType: "",
        courseCode: "",
        courseName: "",
      },
    };
  },
  getters: {
    isSuperAdmin(): boolean {
      return this.userInfo.role === "SUPER_ADMIN";
    },
    isRootOrgAdmin(): boolean {
      return this.userInfo.role === "ROOT_ORG_ADMIN";
    },
    isOrgAdmin(): boolean {
      return this.userInfo.role === "ORG_ADMIN";
    },
    isGreaterThanEqualRootOrgAdmin(): boolean {
      return this.isSuperAdmin || this.isRootOrgAdmin;
    },
    isGreaterThanEqualOrgAdmin(): boolean {
      return this.isSuperAdmin || this.isRootOrgAdmin || this.isOrgAdmin;
    },
    isCourseAdmin(): boolean {
      return this.userInfo.role === "COURSE_ADMIN";
    },
  },
  actions: {
    setUserInfo(res: any) {
      this.$patch({
        userInfo: res,
      });
      setToken(res.token);
      setSessionId(res.identity);
    },
    setPaperAnalysisDetail(res: any) {
      this.$patch({
        paperAnalysisDetail: res,
      });
    },
    resetInfo() {
      this.$reset();
    },
  },
  persist: {
    storage: sessionStorage,
  },
});

export default useMainStore;
